# -*- coding:utf-8 -*-
"""
返回数据接口
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import baostock.common.contants as cons
import pandas as pd
import json
import zlib
import baostock.data.messageheader as msgheader
import baostock.util.socketutil as sock


class ResultData(object):
    def __init__(self):
        """初始化方法"""

        # 消息头
        self.version = cons.BAOSTOCK_CLIENT_VERSION
        self.msg_type = 0  # 消息类型
        self.msg_body_length = 0

        # 消息体
        self.method = ""  # 方法名
        self.user_id = ""  # 用户账号
        self.error_code = cons.BSERR_NO_LOGIN  # 错误代码
        self.error_msg = ""  # 错误代码

        self.cur_page_num = 1  # 当前页码
        self.per_page_count = cons.BAOSTOCK_PER_PAGE_COUNT  # 当前页条数
        self.cur_row_num = 0  # 当前页面遍历条数 V0.5.5

        self.code = ""  # 股票代码
        self.fields = ""  # 指示简称
        self.start_date = ""  # 开始日期
        self.end_date = ""  # 结束日期
        self.frequency = ""  # 数据类型
        self.adjustflag = ""  # 复权类型
        self.data = ""  # 数据值

        self.msg_body = ""  # 消息体

        # self.request_id = 0
        # self.serial_id = 0

    def next(self):
        """ 判断是否还有后续数据
        :return: 有数据返回True，没有数据返回False
        """
        if self.data == "":
            return False

        js_data = json.loads(self.data)  # dict
        list_data = js_data['record']  # list
        if self.cur_row_num < len(list_data):
            # 当前页还有数据
            return True
        else:
            # 当前页没有数据，取下一页数据
            msg_body_split = self.msg_body.split(cons.MESSAGE_SPLIT)

            if self.cur_page_num.isdigit():
                next_page = 0
                next_page = int(self.cur_page_num) + 1
                msg_body_split[2] = str(next_page)
            else:
                print("当前页面编号不正确，请检查后再试")
                return False

            msg_body = cons.MESSAGE_SPLIT.join(msg_body_split)
            msg_header = msgheader.to_message_header(
                    self.msg_type, len(msg_body))

            head_body = msg_header + msg_body
            crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
            receive_data = sock.send_msg(
                        head_body + cons.MESSAGE_SPLIT + str(crc32str))

            if receive_data is None or receive_data.strip() == "":
                return False

            msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
            msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]

            header_arr = msg_header.split(cons.MESSAGE_SPLIT)
            body_arr = msg_body.split(cons.MESSAGE_SPLIT)
            # data.version = header_arr[0]
            # self.msg_type = header_arr[1]
            self.msg_body_length = header_arr[2]

            self.error_code = body_arr[0]
            self.error_msg = body_arr[1]

            if cons.BSERR_SUCCESS == self.error_code:
                self.method = body_arr[2]
                self.user_id = body_arr[3]

                self.cur_page_num = body_arr[4]
                self.per_page_count = body_arr[5]
                self.data = body_arr[12]
                self.cur_row_num = 0
                if self.data == "":
                    return False
                else:
                    return True
            else:
                return False

    def get_row_data(self):
        """获取当前行数据"""

        # 如果数据为空
        if self.data == "":
            return pd.DataFrame()

        # 对数据进行格式化
        js_data = json.loads(self.data)  # dict
        list_data = js_data['record']  # list

        # 对列名进行拆分
        field_arr = self.fields.split(cons.ATTRIBUTE_SPLIT)
        # 去除空格
        i = 0
        while i < len(field_arr):
            field_arr[i] = field_arr[i].strip()
            i += 1

        # 组织返回数据
        if self.cur_row_num < len(list_data):
            df = pd.DataFrame([list_data[self.cur_row_num]], columns=field_arr)
            self.cur_row_num = self.cur_row_num + 1
        else:
            df = pd.DataFrame()
        return df

    def get_data(self):
        """对返回结果进行处理
        :return:DataFrame类型
        """

        if self.data == "":
            return pd.DataFrame()

        # 对数据进行格式化
        js_data = json.loads(self.data)
        # 对列名进行拆分
        field_arr = self.fields.split(cons.ATTRIBUTE_SPLIT)

        i = 0
        while i < len(field_arr):
            # 去除空格
            field_arr[i] = field_arr[i].strip()
            i += 1
        # 组织返回数据
        df = pd.DataFrame(js_data['record'], columns=field_arr)
        self.cur_row_num = len(js_data['record'])
        return df



# -*- coding:utf-8 -*-
"""
常量类
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
# 版本信息
BAOSTOCK_CLIENT_VERSION = "00.5.10"
BAOSTOCK_AUTHOR = "baostock.com"
BAOSTOCK_SERVER_IP = "localhost"  # localhost  www.baostock.com
BAOSTOCK_SERVER_PORT = 10030


BAOSTOCK_PER_PAGE_COUNT = 10000  # 默认每页查询条数


MESSAGE_SPLIT = "\1"  # 消息内部的分隔符

DELIMITER = "\n"  # 分隔符号,消息与消息之间的分隔符

# 参数各属性间的分隔符，如queryHistoryKData中的fields
ATTRIBUTE_SPLIT = ","
# 消息头中，消息体长度，占的位数
MESSAGE_HEADER_BODYLENGTH = 10
# message头长度
MESSAGE_HEADER_LENGTH = 21

# 以下为消息请求类型
# 登陆请求
MESSAGE_TYPE_LOGIN_REQUEST = "00"

# 登陆响应
MESSAGE_TYPE_LOGIN_RESPONSE = "01"

# 登出请求

# 登出响应
MESSAGE_TYPE_LOGOUT_RESPONSE = "03"
MESSAGE_TYPE_LOGOUT_REQUEST = "02"

# 错误信息
MESSAGE_TYPE_EXCEPTIONS = "04"

# 获取历史K线数据请求
MESSAGE_TYPE_GETKDATA_REQUEST = "11"

# 获取历史K线数据响应
MESSAGE_TYPE_GETKDATA_RESPONSE = "12"

# 以上为消息请求类型


# 方法名及包名的对应关系
MESSAGE_PACKAGE_MAPPING = {
    "__query_history_k_data_page": "baostock.security.history"

}

# 以下是错误代码
# 正确返回值
BSERR_SUCCESS = "0"

# 用户未登陆
BSERR_NO_LOGIN = "10001001"

# 用户名或密码错误
BSERR_USERNAMEORPASSWORD_ERR = "10001002"

# 获取用户信息失败
BSERR_GETUSERINFO_FAIL = "10001003"

# 客户端版本号过期
BSERR_CLIENT_VESION_EXPIRE = "10001004"

# 账号登陆数达到上限
BSERR_LOGIN_COUNT_LIMIT = "10001005"

# 用户权限不足
BSERR_ACCESS_INSUFFICIENCE = "10001006"

# 需要登录激活
BSERR_NEED_ACTIVATE = "10001007"


BSERR_USERNAME_EMPTY = "10001008"  # 用户名为空


BSERR_PASSWORD_EMPTY = "10001009"  # 密码为空


BSERR_LOGOUT_FAIL = "10001010"  # 用户登出失败

# 用户未登录
ESERR_NOT_LOGIN = "10001011"


# 网络错误
BSERR_SOCKET_ERR = "10002001"

# 网络连接失败
BSERR_CONNECT_FAIL = "10002002"

# 网络连接超时
BSERR_CONNECT_TIMEOUT = "10002003"

# 网络接收时连接断开
BSERR_RECVCONNECTION_CLOSED = "10002004"

# 网络发送失败
BSERR_SENDSOCK_FAIL = "10002005"

# 网络发送超时
BSERR_SENDSOCK_TIMEOUT = "10002006"

# 网络接收错误
BSERR_RECVSOCK_FAIL = "10002007"

# 网络接收超时
BSERR_RECVSOCK_TIMEOUT = "10002008"

# 解析数据错误
BSERR_PARSE_DATA_ERR = "10004001"

# gzip 解压失败
BSERR_UNGZIP_DATA_FAIL = "10004002"

# 客户端未知错误
BSERR_UNKNOWN_ERR = "10004003"

# 数组越界
BSERR_OUTOF_BOUNDS = "10004004"

# 传入参数为空
BSERR_INPARAM_EMPTY = "10004005"

# 参数错误
BSERR_PARAM_ERR = "10004006"

# 起始日期格式不正确
BSERR_START_DATE_ERR = "10004007"


BSERR_END_DATE_ERR = "10004008"  # 截止日期格式不正确


BSERR_START_BIGTHAN_END = "10004009"  # 起始日期大于终止日期


BSERR_DATE_ERR = "10004010"  # 日期格式不正确

BSERR_CODE_INVALIED = "10004011"  # 无效的证券代码

BSERR_INDICATOR_INVALIED = "10004012"  # 无效的指标

# 超出日期支持范围
BSERR_BEYOND_DATE_SUPPORT = "10004013"

# 不支持的混合证券品种
BSERR_MIXED_CODES_MARKET = "10004014"

# 不支持的证券代码品种
BSERR_NO_SUPPORT_CODES_MARKET = "10004015"

# 交易条数超过上限
BSERR_ORDER_TO_UPPER_LIMIT = "10004016"

# 不支持的交易信息
BSERR_NO_SUPPORT_ORDERINFO = "10004017"

# 指标重复
BSERR_INDICATOR_REPEAT = "10004018"

# 消息格式不正确
BSERR_MESSAGE_ERROR = "10004019"

# 错误的消息类型
BSERR_MESSAGE_CODE_ERROR = "10004020"

# 系统级别错误
BSERR_SYSTEM_ERROR = "10005001"

# 以上是错误代码


# 客户端参数错误
CLIENT_ERROR_PARAM = "参数错误，请检查。"



# -*- coding:utf-8 -*-
"""
字符串方法
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import datetime


def add_zero_for_string(content, length, direction):
    """
    在str的左或右添加0
    :param str:待修改的字符串
    :param length:总共的长度
    :param direction:方向，True左，False右
    :return:
    """
    content = str(content)
    str_len = len(content)
    if str_len < length:
        while str_len < length:
            if direction:
                content = "0" + content
            else:
                content = content + "0"

            str_len = len(content)
    return content

def is_valid_date(str):
    """
    判断是否是一个有效的日期字符串
    :param str:
    :return: 符合格式返回True,
    """
    try:
        datetime.datetime.strptime(str, "%Y-%m-%d")
        return True
    except:
        return False



# -*- coding:utf-8 -*-
"""
发布方法
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import baostock.common.contants as cons
__version__ = cons.BAOSTOCK_CLIENT_VERSION
__author__ = cons.BAOSTOCK_AUTHOR

# login/logout
from baostock.login.loginout import (login, logout)

# history data
from baostock.security.history import (query_history_k_data)



# -*- coding:utf-8 -*-
"""
封装消息接口
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import baostock.common.contants as cons


class MessageHeader(object):

    def __init__(self, msg_type, total_msg_length):
        self.version = cons.BAOSTOCK_CLIENT_VERSION
        self.msg_type = msg_type
        self.total_msg_length = total_msg_length

    def to_message_header_msg(self):
        return self.version + cons.MESSAGE_SPLIT + self.msg_type \
                + cons.MESSAGE_SPLIT + self.total_msg_length




# -*- coding:utf-8 -*-
"""
封装消息头
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import baostock.common.contants as cons
import baostock.util.stringutil as strutil


def to_message_header(msg_type, total_msg_length):

    return_str = cons.BAOSTOCK_CLIENT_VERSION + cons.MESSAGE_SPLIT + msg_type \
                 + cons.MESSAGE_SPLIT \
                 + strutil.add_zero_for_string(total_msg_length, 10, True)
    return return_str



# -*- coding:utf-8 -*-
"""
获取默认socket
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import socket
import baostock.common.contants as cons
import baostock.common.context as context

class SocketUtil(object):
    """Socket工具类"""
    # 记录第一个被创建对象的引用
    instance = None
    # 记录是否执行过初始化动作
    init_flag = False

    def __new__(cls, *args, **kwargs):
        # 1. 判断类属性是否是空对象
        if cls.instance is None:
            # 2. 调用父类的方法，为第一个对象分配空间
            cls.instance = super().__new__(cls)
        # 3. 返回类属性保存的对象引用
        return cls.instance

    def __init__(self):
        SocketUtil.init_flag = True
    
#         # 1. 判断是否执行过初始化动作
#         if SocketUtil.init_flag:
#             return
#         # 2. 如果没有执行过，在执行初始化动作
#         self.instance = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#         self.instance.connect((cons.BAOSTOCK_SERVER_IP, cons.BAOSTOCK_SERVER_PORT))
#         # 3. 修改类属性的标记
#         SocketUtil.init_flag = True

    def connect(self):
        """创建连接"""
        try:
            mySockect = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            mySockect.connect((cons.BAOSTOCK_SERVER_IP, cons.BAOSTOCK_SERVER_PORT))
        except Exception:
            print("服务器连接失败，请稍后再试。")
        setattr(context, "default_socket", mySockect)


def get_default_socket():
    """获取默认连接"""
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((cons.BAOSTOCK_SERVER_IP, cons.BAOSTOCK_SERVER_PORT))
    except Exception:
        print("服务器连接失败，请稍后再试。")
        return None
    return sock



def send_msg(msg):
    """发送消息，并接受消息 """
    try:
#         default_socket = get_default_socket()
        default_socket =    getattr(context, "default_socket")
        if default_socket is not None:
            # str 类型 -> bytes 类型
            msg += "\n" #在消息结尾追加“消息之间的分隔符”
            default_socket.send(bytes(msg, encoding='utf-8'))
                        
            receive = ""
            while True:
                recv = default_socket.recv(1024)
                
                temp_data = bytes.decode(recv)
                #判断是否存在消息结束符
                
                
                receive += temp_data
#                 if not temp_data:
#                     break
                if  receive.endswith("\n"):          
                    break
            
            #default_socket.close()
            return receive
        else:
            return None
    except Exception as ex:
        
        print(ex)
        print("接收数据异常，请稍后再试。")



# -*- coding:utf-8 -*-
"""
登录登出
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import zlib
import baostock.util.socketutil as sock
import baostock.data.resultset as rs
import baostock.common.contants as cons
import baostock.data.messageheader as msgheader
import datetime
import baostock.common.context as conx


def login(user_id='anonymous', password='123456', options=0):
    """登录系统
    :param user_id:用户ID
    :param password:密码
    :param options:可选项，00.5.00版本暂未使用
    :return: ResultData()
    """

    data = rs.ResultData()
    if user_id is None or user_id == "":
        print("用户ID不能为空。")
        data.error_msg = "用户ID不能为空。"
        data.error_code = cons.BSERR_USERNAME_EMPTY
        return data

    setattr(conx, "user_id", user_id)

    if password is None or password == "":
        print("密码不能为空。")
        data.error_msg = "密码不能为空。"
        data.error_code = cons.BSERR_PASSWORD_EMPTY
        return data

    # 组织体信息
    msg_body = "login" + cons.MESSAGE_SPLIT + user_id + cons.MESSAGE_SPLIT + \
        password + cons.MESSAGE_SPLIT + str(options)

    # 组织头信息
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_LOGIN_REQUEST, len(msg_body))
    head_body = msg_header + msg_body

    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))

    # 发送并接收消息
    mySocketUtil= sock.SocketUtil()
    #创建连接
    mySocketUtil.connect()
    
    receive_data = sock.send_msg(
        head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]

    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)

    data.msg_type = header_arr[1]
    data.msg_body_length = header_arr[2]

    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        print("login success!")
        data.method = body_arr[2]
        data.user_id = body_arr[3]
    else:
        print("login failed!")

    return data


def logout(user_id='anonymous'):
    """登出系统，默认用户ID：anonymous
    :param user_id:用户ID
    :return:ResultData()
    """

    now_time = datetime.datetime.now().strftime('%Y%m%d%H%M%S')

    # 组织体信息
    msg_body = "logout" + cons.MESSAGE_SPLIT + \
        user_id + cons.MESSAGE_SPLIT + now_time

    # 组织头信息
    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_LOGOUT_REQUEST, len(msg_body))

    head_body = msg_header + msg_body

    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))

    # 发送并接收消息
    receive_data = sock.send_msg(
        head_body + cons.MESSAGE_SPLIT + str(crc32str))

    data = rs.ResultData()

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]

    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)

    data.msg_type = header_arr[1]
    data.msg_body_length = header_arr[2]

    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        print("logout success!")
        data.method = body_arr[2]
        data.user_id = body_arr[3]
    else:
        print("logout failed!")

    if hasattr(conx, "defallt_socket"):
        if getattr(conx, "default_socket") is not None:
            # getattr(conx, "default_socket").close()
            pass
    return data



import baostock as bs
import pandas as pd

# 登陆系统
lg = bs.login(user_id="anonymous", password="123456")
# 显示登陆返回信息
print(lg.error_code)
print(lg.error_msg)
# 详细指标参数，参见“历史行情指标参数”章节
rs = bs.query_history_k_data("sh.601398",
    "date,code,open,high,low,close,volume,amount,adjustflag",
    start_date='2017-01-01', end_date='2017-01-31', 
    frequency="d", adjustflag="3")
print(rs.error_code)
print(rs.error_msg)
# 获取具体的信息
result = pd.DataFrame(columns=["date","code","open","high","low","close","volume","amount","adjustflag"])
while (rs.error_code == '0') & rs.next():
    # 分页查询，将每页信息合并在一起
    result = result.append(rs.get_row_data(), ignore_index=True)
result.to_csv("D:\\history_k_data.csv", index=False)
print(result)
# 登出系统
bs.logout()



# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

long_desc = """
baoshare
===============
    
* It's easy to use because most of the data returned are pandas DataFrame objects
* We have our own data server, efficient and stable operation
* Free china stock market data
* Friendly to machine learning and data mining

Target Users
--------------

* China Financial Market Analyst
* Financial data analysis enthusiasts
* Quanters who are interested in china stock market

Installation
--------------

    pip install baoshare

Upgrade
---------------

    pip install baoshare --upgrade

Quick Start
--------------

::

    import baostock as bs
    import pandas as pd
    
    # 登陆系统
    lg = bs.login(user_id="anonymous", password="123456")
    # 显示登陆返回信息
    print(lg.error_code)
    print(lg.error_msg)
    # 详细指标参数，参见“历史行情指标参数”章节
    rs = bs.query_history_k_data("sh.601398",
        "date,code,open,high,low,close,volume,amount,adjustflag",
        start_date='2017-01-01', end_date='2017-01-31', 
        frequency="d", adjustflag="3")
    print(rs.error_code)
    print(rs.error_msg)
    # 获取具体的信息
    result = pd.DataFrame(columns=["date","code","open","high","low","close","volume","amount","adjustflag"])
    while (rs.error_code == '0') & rs.next():
        # 分页查询，将每页信息合并在一起
        result = result.append(rs.get_row_data(), ignore_index=True)
    result.to_csv("D:\\history_k_data.csv", index=False)
    print(result)
    # 登出系统
    bs.logout()

return::

              date       code    open    high     low   close preclose     volume
    0   2017-01-03  sh.601398  4.4000  4.4300  4.3900  4.4300   4.4100  104161632   
    1   2017-01-04  sh.601398  4.4200  4.4400  4.4100  4.4300   4.4300  118923425   
    2   2017-01-05  sh.601398  4.4300  4.4500  4.4200  4.4400   4.4300   87356137   
    3   2017-01-06  sh.601398  4.4400  4.4500  4.4300  4.4400   4.4400   87008191   
    4   2017-01-09  sh.601398  4.4500  4.4800  4.4300  4.4600   4.4400  117454094   
    5   2017-01-10  sh.601398  4.4500  4.4700  4.4400  4.4600   4.4600   63663257   
    6   2017-01-11  sh.601398  4.4600  4.4800  4.4500  4.4700   4.4600   52395427   
    7   2017-01-12  sh.601398  4.4600  4.4700  4.4400  4.4700   4.4700   62166279    
    
                 amount adjustflag      turn tradestatus  
    0    460087744.0000          3  0.038634           1  
    1    526408816.0000          3  0.044109           1  
    2    387580736.0000          3  0.032401           1  
    3    386138112.0000          3  0.032272           1  
    4    523539392.0000          3  0.043564           1  
    5    283646224.0000          3  0.023613           1  
    6    233898107.0000          3  0.019434           1  
    7    277258304.0000          3  0.023058           1  

"""


setup(
    name='baoshare',
    version='0.5.4',
    description=(
        'A tool for obtaining historical data of China stock market'
    ),
    long_description=long_desc,
    author='baoshare',
    author_email='baoshare@163.com',
    license='BSD License',
    packages=find_packages(),
    platforms=["all"],
    install_requires=[
        'pandas>=0.18.0'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',   # 开发状态
        'Environment :: Console', # 运行环境
        'License :: OSI Approved :: BSD License', # BSD协议
        'Operating System :: OS Independent', # 与平台无关
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)



# -*- coding:utf-8 -*-
"""
获取历史行情
@author: baostock.com
@group : baostock.com
@contact: baostock@163.com
"""
import datetime
import time
import zlib

import baostock.common.contants as cons
import baostock.common.context as conx
import baostock.data.messageheader as msgheader
import baostock.data.resultset as rs
import baostock.util.socketutil as sock
import baostock.util.stringutil as strUtil



def query_history_k_data(code, fields, start_date=None, end_date=None,
                         frequency='d', adjustflag='3'):
    """获取历史K线"""
    return __query_history_k_data_page(1, cons.BAOSTOCK_PER_PAGE_COUNT, code, fields, start_date,
                                       end_date, frequency, adjustflag)


def __query_history_k_data_page(cur_page_num, per_page_count, code, fields,
                                start_date, end_date, frequency, adjustflag):
    """获取历史K线，私有方法"""
    data = rs.ResultData()
    if code is None or code == "":
        print("股票代码不能为空，请检查。")
        data.error_msg = "股票代码不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if len(code) != 9:
        print("股票代码格式错误，请检查。")
        data.error_msg = "股票代码格式错误，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if fields is None or fields == "":
        data.error_msg = "指示简称不能为空，请检查。"
        data.error_code = cons.BSERR_PARAM_ERR
        print("指示简称不能为空，请检查。")
        return data

    if start_date is None or start_date == "":
        start_date = time.strftime("%Y-%m-%d", time.localtime())
    if end_date is None or end_date == "":
        end_date = time.strftime("%Y-%m-%d", time.localtime())

    if start_date != "" and end_date != "":
        if strUtil.is_valid_date(start_date) and strUtil.is_valid_date(end_date):
            start_date_time = datetime.datetime.strptime(
                start_date, '%Y-%m-%d')
            end_date_time = datetime.datetime.strptime(end_date, '%Y-%m-%d')
            if end_date_time < start_date_time:
                print("起始日期大于终止日期，请修改。")
                data.error_code = cons.BSERR_START_BIGTHAN_END
                data.error_msg = "起始日期大于终止日期，请修改。"
                return data
        else:
            print("日期格式不正确，请修改。")
            return

    if frequency is None or frequency == "":
        print("频率（frequency）不可为空。")
        data.error_msg = "频率（frequency）不可为空"
        data.error_code = cons.BSERR_PARAM_ERR
        return data
    if adjustflag is None or adjustflag == "":
        print("复权类型（adjustflag）不可为空")
        data.error_msg = "复权类型（adjustflag）不可为空"
        data.error_code = cons.BSERR_PARAM_ERR
        return data

    user_id = ""
    try:
        user_id = getattr(conx, "user_id")
    except Exception:
        print("用户未登陆，请先登录。")
        data.error_code = cons.BSERR_NO_LOGIN
        data.error_msg = "用户未登陆，请先登录。"
        return data

    msg_body = "__query_history_k_data_page" + cons.MESSAGE_SPLIT + user_id + cons.MESSAGE_SPLIT \
               + str(cur_page_num) + cons.MESSAGE_SPLIT + str(per_page_count) + cons.MESSAGE_SPLIT + code \
               + cons.MESSAGE_SPLIT + fields + cons.MESSAGE_SPLIT + start_date \
               + cons.MESSAGE_SPLIT + end_date + cons.MESSAGE_SPLIT + frequency \
               + cons.MESSAGE_SPLIT + adjustflag

    msg_header = msgheader.to_message_header(
        cons.MESSAGE_TYPE_GETKDATA_REQUEST, len(msg_body))

    data.msg_type = cons.MESSAGE_TYPE_GETKDATA_REQUEST
    data.msg_body = msg_body

    head_body = msg_header + msg_body
    crc32str = zlib.crc32(bytes(head_body, encoding='utf-8'))
    
    receive_data = sock.send_msg(head_body + cons.MESSAGE_SPLIT + str(crc32str))

    if receive_data is None or receive_data.strip() == "":
        data.error_code = cons.BSERR_RECVSOCK_FAIL
        data.error_msg = "网络接收错误。"
        return data

    msg_header = receive_data[0:cons.MESSAGE_HEADER_LENGTH]
    msg_body = receive_data[cons.MESSAGE_HEADER_LENGTH:-1]

    header_arr = msg_header.split(cons.MESSAGE_SPLIT)
    body_arr = msg_body.split(cons.MESSAGE_SPLIT)

    # data.version = header_arr[0]
    # data.msg_type = header_arr[1]
    data.msg_body_length = header_arr[2]

    data.error_code = body_arr[0]
    data.error_msg = body_arr[1]

    if cons.BSERR_SUCCESS == data.error_code:
        data.method = body_arr[2]
        data.user_id = body_arr[3]
        data.cur_page_num = body_arr[4]
        data.per_page_count = body_arr[5]
        data.code = body_arr[6]
        data.fields = body_arr[7]
        data.start_date = body_arr[8]
        data.end_date = body_arr[9]
        data.frequency = body_arr[10]
        data.adjustflag = body_arr[11]
        data.data = body_arr[12]

    return data
